#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include <time.h>
#include <mpi.h>

double *create_rand_nums(int num_elements, int rank) {
  double *rand_nums = (double *)malloc(sizeof(double) * num_elements);
  
  for (int i = 0; i < num_elements; i++)
    rand_nums[i] = ((double)rank * rand() / RAND_MAX) + rank;
  
  return rand_nums;
}

double find_min(double *array, int num_elements) {
  double min = DBL_MAX;

  for (int i = 0; i < num_elements; ++i) {
    min = (array[i] < min) ? array[i] : min; 
  }

  return min;
}

double find_max(double *array, int num_elements) {
  double max = DBL_MIN;

  for (int i = 0; i < num_elements; ++i) {
    max = (array[i] > max) ? array[i] : max; 
  }

  return max;
}

int main(int argc, char** argv) {
  int nelems_pp = atoi(argv[1]);

  MPI_Init(&argc, &argv);

  int rank, world_size, gath_root = 0;
  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
  MPI_Comm_size(MPI_COMM_WORLD, &world_size);

  srand(time(NULL) + rank);

  MPI_Request* min_request = (MPI_Request*)malloc(sizeof(MPI_Request) * world_size);
  MPI_Request* max_request = (MPI_Request*)malloc(sizeof(MPI_Request) * world_size);

  double* remote_min, *remote_max;
  if(rank == 0) {
    remote_min = (double*)malloc(sizeof(double) * world_size);
    remote_max = (double*)malloc(sizeof(double) * world_size);
  }

  double* rand_nums = create_rand_nums(nelems_pp, rank);
  
  double pmin = find_min(rand_nums, nelems_pp);

  printf("Process with rank %d: min = %lf.\n", rank, pmin);

  if (rank == 0) {
    for (int i = 1; i < world_size; i++) {
       MPI_Irecv(&remote_min[i], 1, MPI_DOUBLE, i, 0, MPI_COMM_WORLD, &min_request[i]);
    }
  } else {
    MPI_Isend(&pmin, 1, MPI_DOUBLE, 0, 0, MPI_COMM_WORLD, &min_request[0]);
  }

  double pmax = find_max(rand_nums, nelems_pp);

  printf("Process with rank %d: max = %lf.\n", rank, pmax);

  if (rank == 0) {
    for (int i = 1; i < world_size; i++) {
       MPI_Irecv(&remote_max[i], 1, MPI_DOUBLE, i, 0, MPI_COMM_WORLD, &max_request[i]);
    }
  } else {
    MPI_Isend(&pmax, 1, MPI_DOUBLE, 0, 0, MPI_COMM_WORLD, &max_request[0]);
  }

  if(rank == 0) {
    for (int i = 1; i < world_size; ++i) {
      MPI_Wait(&min_request[i], NULL);

      if ( remote_min[i] < pmin )
        pmin = remote_min[i];
    }

    for (int i = 1; i < world_size; ++i) {
      MPI_Wait(&max_request[i], NULL);

      if ( remote_max[i] > pmax )
        pmax = remote_max[i];
    }

    free(remote_min);
    free(remote_max);

    printf("min = %lf, max = %lf.\n", pmin, pmax);
  } else {  
    MPI_Wait(&min_request[0], NULL);
    MPI_Wait(&max_request[0], NULL);
  }

  free(rand_nums);
  free(min_request);
  free(max_request);

  MPI_Finalize();

  return 0;
}