#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>

#define NSTEPS 1000
#define N 100000

int main (int argc, char *argv[]) {
  int rank, size;
  int right_rank, left_rank;
  int n;
  
  double* u;
  double* u_prev;

  const double alpha = 0.1;

  MPI_Request requests[4];
  MPI_Status statuses[4];

  MPI_Init(&argc, &argv);

  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
  MPI_Comm_size(MPI_COMM_WORLD, &size);

  right_rank = (rank+1     ) % size;
  left_rank  = (rank-1+size) % size;

  n = N / size;
  if (rank < N % size)
    n += 1;

       u = malloc((n+2)*sizeof(double));
  u_prev = malloc((n+2)*sizeof(double));

  for(int i = 0; i <= n+1; ++i){
    u[i] = 0.0;
    u_prev[i] = 0.0;
  }

  if (rank == 0) {
    u[1] = 100.0;
    u_prev[1] = 100.0;
  }

  if (rank == size-1) {
    u[n] = 100.0;
    u_prev[n] = 100.0;
  }

  for(int step = 0; step < NSTEPS; ++step) {
    MPI_Irecv(&u_prev[0] , 1, MPI_DOUBLE, left_rank, 10, MPI_COMM_WORLD, &requests[0]);
    MPI_Irecv(&u_prev[n+1], 1, MPI_DOUBLE, right_rank, 20, MPI_COMM_WORLD, &requests[1]);
    
    MPI_Isend(&u_prev[1] , 1, MPI_DOUBLE, left_rank, 20, MPI_COMM_WORLD, &requests[2]);
    MPI_Isend(&u_prev[n] , 1, MPI_DOUBLE, right_rank, 10, MPI_COMM_WORLD, &requests[3]);

    for (int i = 2; i < n; ++i)
      u[i] = u_prev[i] + alpha * (u_prev[i-1] - 2 * u_prev[i] + u_prev[i+1]);

    MPI_Waitall(4, requests, statuses);

    u[1] = u_prev[1] + alpha * (u_prev[0] - 2 * u_prev[1] + u_prev[2]);
    u[n] = u_prev[n] + alpha * (u_prev[n-1] - 2 * u_prev[n] + u_prev[n+1]);

    double* temp = u; u = u_prev; u_prev = temp;
  }

  free(u);
  free(u_prev);

  MPI_Finalize();

  return 0;
}