#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <mpi.h>

int main(int argc, char** argv) {
  const int nelems_per_rank = 1000;

  double local_sum, global_sum, mean;
  double local_sq_diff, global_sq_diff;

  int rank, size;

  local_sum = local_sq_diff = 0;

  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
  MPI_Comm_size(MPI_COMM_WORLD, &size);

  srand(time(NULL) * rank);
  
  double *values = (double *)malloc(sizeof(double) * nelems_per_rank);
  for (int i = 0; i < nelems_per_rank; ++i)
    values[i] = (rand() / (double)RAND_MAX);
  
  for (int i = 0; i < nelems_per_rank; ++i)
    local_sum += values[i];

  MPI_Allreduce(&local_sum, &global_sum, 1, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
  mean = global_sum / (nelems_per_rank * size);

  for (int i = 0; i < nelems_per_rank; ++i)
    local_sq_diff += (values[i] - mean) * (values[i] - mean);

  MPI_Reduce(&local_sq_diff, &global_sq_diff, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);


  if (rank == 0) {
    double stddev = sqrt(global_sq_diff / (nelems_per_rank * size));
    printf("Mean = %lf, Standard deviation = %lf\n", mean, stddev);
  }

  free(values);

  MPI_Finalize();

  return 0;
}