    program main
      use mpi

      implicit none

      integer, parameter :: nelems_per_rank = 1000

      double precision :: local_sum, global_sum, mean
      double precision :: local_sq_diff, global_sq_diff
      double precision :: stddev

      double precision, dimension(:), allocatable :: values
      
      integer :: rank, size, ierror, i

      call MPI_Init(ierror)
      call MPI_Comm_rank(MPI_COMM_WORLD, rank, ierror)
      call MPI_Comm_size(MPI_COMM_WORLD, size, ierror)

      call random_seed()
      
      allocate(values(nelems_per_rank))
      call random_number(values)

      local_sum = 0.0
      local_sq_diff = 0.0

      do i = 1, nelems_per_rank
        local_sum = local_sum + values(i)
      enddo

      call MPI_Allreduce(local_sum, global_sum, 1, MPI_DOUBLE_PRECISION, MPI_SUM, MPI_COMM_WORLD, ierror)
      mean = global_sum / (nelems_per_rank * size)

      do i = 1, nelems_per_rank
        local_sq_diff = local_sq_diff + (values(i) - mean) * (values(i) - mean)
      end do

      call MPI_Reduce(local_sq_diff, global_sq_diff, 1, MPI_DOUBLE_PRECISION, MPI_SUM, 0, MPI_COMM_WORLD, ierror)

      if (rank .eq. 0) then
        stddev = dsqrt(global_sq_diff / dble(nelems_per_rank * size))
        print 100, mean, stddev
100     format('Mean = ', f12.6, ', Standard deviation = ', f12.6, f12.6)
      end if

      deallocate(values)

      call MPI_Finalize(ierror)
    end program